@@ -30,6 +30,9 @@
 #include <sys/time.h>
 #include <ctype.h>
 
+#ifdef ALPHA
+#define FREEBSD
+#endif
 #ifdef AIXV3
 #include <sys/select.h>
 #endif   
@@ -66,8 +69,8 @@
 #define	VWERASE		VWERSE
 #endif
 
-/* for greek elot-928 keyboard mapping */
-#ifdef GREEK_ELOT_KBD	
+/* for greek keyboard mapping */
+#ifdef GREEK_KBD	
 #include "grkelot.h"
 
 static greek_mode_ON = False;
@@ -126,8 +129,9 @@ KeySym BigFontKeysym = XK_greater;
 KeySym SmallFontKeysym = XK_less;
 KeySym PageUpKeysym = XK_Prior;
 KeySym PageDownKeysym = XK_Next;
-#ifdef GREEK_ELOT_KBD
+#ifdef GREEK_KBD
 KeySym GreekSwitchKeysym = XK_Mode_switch;
+int GreekMode = GREEK_ELOT928;
 #endif
 
 int refresh_nl_count=0;
@@ -240,7 +244,7 @@ static void catch_sig(int sig)
 static int run_command(unsigned char *command,unsigned char **argv)
 {
   int ptyfd;
-  int uid, gid;
+  int uid, gid,shpgrp;
   unsigned char *s3, *s4;
   int i;
   int width, height;
@@ -252,6 +256,7 @@ static int run_command(unsigned char *command,unsigned char **argv)
 
   /*  First find a master pty that we can open.  
    */
+
 #ifdef SVR4
   ptyfd = open("/dev/ptmx",O_RDWR);
   if (ptyfd < 0) 
@@ -295,7 +300,10 @@ static int run_command(unsigned char *command,unsigned char **argv)
   for (i = 1; i <= 15; i++)
     signal(i,catch_sig);
   signal(SIGCHLD,catch_child);
+  lstat(ttynam,&ttyfd_stat);
+
   makeutent(&ttynam[5]);	/* stamp /etc/utmp */  
+
   comm_pid = fork();
   if (comm_pid < 0) 
     {
@@ -306,9 +314,6 @@ static int run_command(unsigned char *command,unsigned char **argv)
     {
       struct group *gr;
       
-      if (setsid() < 0)
-	error("failed to set process group");
-      
       if ((ttyfd = open(ttynam,O_RDWR)) < 0) 
 	{
 	  error("could not open slave tty %s",ttynam);
@@ -318,9 +323,6 @@ static int run_command(unsigned char *command,unsigned char **argv)
       ioctl(ttyfd,I_PUSH,"ptem");
       ioctl(ttyfd,I_PUSH,"ldterm");
 #endif
-#if defined(TIOCSCTTY)
-      ioctl(ttyfd, TIOCSCTTY, 0) ;
-#endif
 
       uid = getuid();
 #ifndef SVR4
@@ -328,7 +330,6 @@ static int run_command(unsigned char *command,unsigned char **argv)
 	gid = gr->gr_gid;
       else
 	gid = -1;
-      fstat(ttyfd,&ttyfd_stat);
       fchown(ttyfd,uid,gid);
       fchmod(ttyfd,0600);
 #endif
@@ -357,6 +358,19 @@ static int run_command(unsigned char *command,unsigned char **argv)
       if (ttyfd > 2)
 	close(ttyfd);
       
+      if (setsid() < 0)
+	perror("failed to set process group");
+#if defined(TIOCSCTTY)
+      ioctl(0, TIOCSCTTY, 0) ;
+#endif
+      {
+	int pgrp = getpid();
+	ioctl(0, TIOCSPGRP, (char *)&pgrp);
+	setpgrp();
+	close(open(ttynam, O_WRONLY, 0));
+	setpgrp ();
+      }
+
       /* init of termios structure		*/
 #ifdef FREEBSD
       ioctl(0,TIOCGETA,(char *)&ttmode);
@@ -368,7 +382,6 @@ static int run_command(unsigned char *command,unsigned char **argv)
 #   endif        
 #endif
  
-
 #ifdef HPUX
       ttmode.c_iflag = BRKINT | IGNPAR | ICRNL| IXON;
       ttmode.c_lflag = ISIG|IEXTEN|ICANON|ECHO|ECHOE|ECHOK;
@@ -380,7 +393,11 @@ static int run_command(unsigned char *command,unsigned char **argv)
       ttmode.c_cflag = B9600 | CS8 | CREAD;
 
       ttmode.c_cc[VEOF] = CEOF;
+#ifdef ALPHA
+      (unsigned) ttmode.c_cc[VEOL] = CEOL;
+#else
       ttmode.c_cc[VEOL] = CEOL; 
+#endif
       ttmode.c_cc[VINTR] = CINTR;
       ttmode.c_cc[VQUIT] = CQUIT;
       ttmode.c_cc[VERASE] = CERASE;
@@ -421,7 +438,6 @@ static int run_command(unsigned char *command,unsigned char **argv)
       ttmode.c_cc[VTIME] = 0;
 #endif
 
-
       if(mask == 0x7f)
 	ttmode.c_cflag = B9600 | PARENB | CS7 | CREAD;
 #ifdef FREEBSD
@@ -435,8 +451,6 @@ static int run_command(unsigned char *command,unsigned char **argv)
 #endif
       scr_get_size(&width,&height);
       tty_set_size(0,width,height);
-      setgid(getgid());
-      setuid(uid);
       if(login_shell)
 	{
           char *tmp;
@@ -448,6 +462,9 @@ static int run_command(unsigned char *command,unsigned char **argv)
 	  strcat(argv0,tmp);
 	  argv[0] = argv0;
 	}
+      setgid(getgid());
+      setuid(uid);
+
       execvp(command,(char **)argv);
       error("Couldn't execute %s",command);
       clean_exit(1);
@@ -475,8 +492,8 @@ void tty_set_size(int fd,int width,int height)
  */
 void init_command(unsigned char *command,unsigned char **argv)
 {
-#ifdef GREEK_ELOT_KBD
-  greek_init();
+#ifdef GREEK_KBD
+  greek_init(GreekMode);
   /*atexit(greek_end);  not necessary */
 #endif          
   /*  Enable the delete window protocol.
@@ -547,7 +564,7 @@ static unsigned char *lookup_key(XEvent *ev,int *pcount)
       count = 0;
     }
 #endif
-#ifdef GREEK_ELOT_KBD
+#ifdef GREEK_KBD
   else if(keysym == GreekSwitchKeysym)
     {
       greek_mode_ON = !greek_mode_ON;
@@ -760,7 +777,7 @@ static unsigned char *lookup_key(XEvent *ev,int *pcount)
       strcpy(kbuf,"\033[6~");
       count = 4;
       break;
-#ifdef GREEK_ELOT_KBD
+#ifdef GREEK_KBD
     default:
       if (greek_mode_ON)
         count = greek_xlat(kbuf, count);
@@ -996,7 +1013,7 @@ void get_X_event(void)
       size_set = 1;
       return;
     case SelectionClear:
-      scr_clear_selection();
+      scr_delete_selection();
       return;
     case SelectionNotify:
       scr_paste_primary(event.xselection.requestor,
@@ -1011,15 +1028,22 @@ void get_X_event(void)
     case ButtonPress:
       if (event.xany.window == vt_win)
 	{
-	  switch (event.xbutton.button)
+	  if(event.xbutton.subwindow != None)
 	    {
-	    case Button1 :
-	      scr_start_selection(event.xbutton.x,event.xbutton.y);
-	      return;
-	    case Button3 :
-	      scr_extend_selection(event.xbutton.x,event.xbutton.y);
-	    default:
-	      return;
+	      Gr_ReportButtonPress(event.xbutton.x, event.xbutton.y);
+	    }
+	  else
+	    {
+	      switch (event.xbutton.button)
+		{
+		case Button1 :
+		  scr_start_selection(event.xbutton.x,event.xbutton.y);
+		  return;
+		case Button3 :
+		  scr_extend_selection(event.xbutton.x,event.xbutton.y);
+		default:
+		  return;
+		}
 	    }
 	}
       if (event.xany.window == sbar.sb_win)
@@ -1065,16 +1089,23 @@ void get_X_event(void)
     case ButtonRelease :
       if (event.xany.window == vt_win)
 	{
-	  switch (event.xbutton.button)
+	  if(event.xbutton.subwindow != None)
 	    {
-	    case Button1 :
-	    case Button3 :
-	      scr_make_selection(event.xbutton.time);
-	      return;
-	    case Button2 :
-	      scr_request_selection(event.xbutton.time,event.xbutton.x,
-				    event.xbutton.y);
-	      return;
+	      Gr_ReportButtonRelease(event.xbutton.x, event.xbutton.y);
+	    }
+	  else
+	    {
+	      switch (event.xbutton.button)
+		{
+		case Button1 :
+		case Button3 :
+		  scr_make_selection(event.xbutton.time);
+		  return;
+		case Button2 :
+		  scr_request_selection(event.xbutton.time,event.xbutton.x,
+					event.xbutton.y);
+		  return;
+		}
 	    }
 	}
       return;
@@ -1664,20 +1695,24 @@ void process_sgr_mode(int c,int private,int nargs,int *arg)
 void process_robs_sequence(void)
 {
   unsigned char key,c,c1;
-  int done,i,nargs;
+  int done,i,nargs,negative;
   long args[1000];
   unsigned char text[1000];
 
   nargs = 0;
   done = 0;
   args[nargs] = 0;
+  negative = 0;
   c1 = get_com_char(0);
   while((!done)&&(nargs < 999))
     {
-      while(isdigit(c = get_com_char(0)))
+      while(isdigit(c = get_com_char(0))||(c == '-'))
 	{
+	  if(c == '-')
+	    negative = 1-negative;
 	  args[nargs] = args[nargs]*10+(c-'0');
 	}
+      if(negative)args[nargs] = -args[nargs];
       if(c!= ';')
 	done = 1;
       nargs++;
