@@ -20,12 +20,15 @@
 #ifndef _GRKELOT_H
 #define _GRKELOT_H
 
+#define GREEK_ELOT928	0
+#define GREEK_IBM437	1
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 /* --- Prototypes --- */
-void greek_init(void);
+void greek_init(int greek_mode);
 void greek_end(void);
 void greek_reset(void);
 int greek_xlat(char *s, int num_chars);
