@@ -73,7 +73,7 @@ int selanchor_col=0, selend_col=0;
 int selstartx = 0,selstarty = 0;
 int selection_screen = LOW;
 int selected = SELECTION_CLEAR;
-
+unsigned char *ch = NULL;
 int current_screen = LOW;
 
 /* this is a pointer to an array of size cwidth, with a 1 for any
@@ -121,6 +121,9 @@ extern Colormap	colormap;
 void scr_init(void)
 {
   int i;
+#ifdef DEBUG
+  fprintf(stderr,"init");  
+#endif
 
   for(i=0;i<NSCREENS;i++)
     {
@@ -161,6 +164,17 @@ void scr_reset(void)
   RENDITION *new_rend;
   int i,j,k,l,sl;
 
+#ifdef DEBUG
+  fprintf(stderr,"scr_reset\n");
+#endif
+  if((previous_cols[0] == MyWinInfo.cwidth)&&
+     (previous_cols[1] == MyWinInfo.cwidth)&&
+     (previous_rows[0] == MyWinInfo.cheight + MyWinInfo.saved_lines)&&
+     (previous_rows[1] == MyWinInfo.cheight))
+    return;
+#ifdef DEBUG
+  fprintf(stderr,"scr_reset continue\n");
+#endif
   /* In case rows/columns are invalid */
   if(MyWinInfo.cwidth <= 0)
     MyWinInfo.cwidth = 80;
@@ -276,6 +290,10 @@ void scr_reset(void)
       else
 	tabs[i] = 0;
     }
+  if(ch != NULL)
+    free(ch);
+  ch = (unsigned char *)malloc(MyWinInfo.cwidth + 1);
+
   tty_set_size(comm_fd,MyWinInfo.cwidth,MyWinInfo.cheight);
 }
 
@@ -357,7 +375,7 @@ void scr_backspace(void)
 void scr_change_screen(int direction)
 {
   int x,y,r,i,j;
-  unsigned char t;
+  unsigned int t;
 
   MyWinInfo.offset = 0;
 #ifdef DEBUG
@@ -416,15 +434,19 @@ void scr_change_screen(int direction)
  **************************************************************************/
 void scr_change_rendition(int mode,int style)
 {
+#ifdef DEBUG
+  check_text("change_rend");
+#endif
   if(mode == 0)
     rstyle |= style;
   else
     {
       rstyle &= ~style;
+      rstyle = (rstyle & 0x0ff) | (fore_color <<8) | (back_color <<16);
       if(style == ~0)
 	{
-	  fore_color = 0;
-	  back_color = 0;
+/*	  fore_color = 0;
+	  back_color = 0;*/
 	}
     }
   set_font_style();
@@ -435,6 +457,9 @@ void scr_change_rendition(int mode,int style)
  **************************************************************************/
 void set_font_style(void)
 {
+#ifdef DEBUG
+  check_text("font_style");
+#endif
   rstyle &= ~RS_GRFONT;
   rstyle &= ~RS_GBFONT;
   if(charsets[cScreen->charset] == 'A')
@@ -448,6 +473,10 @@ void set_font_style(void)
  *************************************************************************/
 void scr_get_size(int *width_p,int *height_p)
 {
+
+#ifdef DEBUG
+  check_text("get_size");
+#endif
   *width_p = MyWinInfo.cwidth;
   *height_p = MyWinInfo.cheight;
 }
@@ -458,6 +487,10 @@ void scr_get_size(int *width_p,int *height_p)
  **************************************************************************/
 void scr_focus(int is_in)
 {
+#ifdef DEBUG
+  check_text("focus");
+#endif
+
   focus = is_in;
 }
 
@@ -470,6 +503,11 @@ void scr_secure(void)
   static int state = 0;
   GC tgc;
   unsigned long temp;
+
+#ifdef DEBUG
+  check_text("secure");
+#endif
+
   if(!state)
     {
       if(XGrabKeyboard(display,main_win,True,GrabModeAsync,
@@ -509,6 +547,10 @@ void scr_add_lines(unsigned char *c,int nl_count,int n)
 {
   int i,nl,x;
 
+#ifdef DEBUG
+  check_text("add_lines");
+#endif
+
   if(selected)check_selection(cScreen->row,cScreen->col);
       
   nl = cScreen->row + nl_count - cScreen->bmargin;
@@ -700,7 +742,7 @@ int scroll(int row1,int row2,int count)
  * If (row,col) is within a selected region of text, remove the selection
  *
  ****************************************************************************/
-inline void check_selection(int row, int col)
+inline void  check_selection(int row, int col)
 {
   int val,a,b,c,d;
 #ifdef DEBUG
@@ -1394,6 +1436,29 @@ void scr_clear_selection(void)
 
   selected = SELECTION_CLEAR;
 
+  for(j = 0 ; j < MyWinInfo.saved_lines + MyWinInfo.cheight ;j++)
+    {
+      x=j*(MyWinInfo.cwidth+1);
+      for(i = 0 ; i < (MyWinInfo.cwidth +1) ; i++)
+	cScreen->rendition[x+i] &= ~RS_SELECTED;
+    }  
+
+  for(j = 0 ; j < MyWinInfo.cheight ;j++)
+    {
+      x=j*(MyWinInfo.cwidth+1);
+      for(i = 0 ; i < (MyWinInfo.cwidth +1) ; i++)
+	cScreen->rendition[x+i] &= ~RS_SELECTED;
+    }  
+
+  selanchor_row = selanchor_col = selend_row = selend_col = 0;
+}
+
+void scr_delete_selection(void)
+{
+  int j,x,i;
+
+  selected = SELECTION_CLEAR;
+
   if (selection_text != NULL) 
     {
       safefree(selection_text,"sel_text","clear_sel");
@@ -1667,7 +1732,6 @@ void refresh()
   extern int font_num;
   char force_next,fn;
   int j,k,l,x1,y1,x,n,k0,xrow,x2,xrow2,k1,trow,trow2,count,j0;
-  unsigned char ch[256];
   int rval, fore, back;
   unsigned long newgcm = 0;
   XGCValues newgcv;
@@ -1781,6 +1845,7 @@ void refresh()
     }
   d_xcursor = d_xcursor * (MyWinInfo.cwidth+1)  + cScreen->col;
 
+
   /* For a first cut, do it one character at a time */
   for(j=0;j<(MyWinInfo.cheight);j++)
     {
@@ -1956,6 +2021,10 @@ void scr_set_tab(int value)
 {
   int i;
 
+#ifdef DEBUG
+  check_text("set_tab");  
+#endif
+
   if(value < 0)
     {
       for(i=0;i<MyWinInfo.cwidth;i++)
@@ -1981,6 +2050,7 @@ void check_text(char *a)
   int i;
   static char *previous = "a";
 
+  fprintf(stderr,"%s\n",a);
   for(i=0; i< (MyWinInfo.cheight + MyWinInfo.saved_lines);i++)
     if(cScreen->text[i*(MyWinInfo.cwidth+1)+MyWinInfo.cwidth] != 0)
       {
@@ -1996,14 +2066,18 @@ void check_text(char *a)
 #endif
 
 
-/***************************************************************************************
+/******************************************************************************
  * Toggle reverse video settings 
- **************************************************************************************/
+ *****************************************************************************/
 void scr_rev_vid(int mode)
 {
   static int current = LOW;
   GC t;
 
+#ifdef DEBUG
+  check_text("rev_vid");  
+#endif
+
   if(current == mode)
     return;
   else
@@ -2034,6 +2108,9 @@ scr_fore_color(int color)
 {
   XColor new_color;
 
+#ifdef DEBUG
+  check_text("set_color");  
+#endif
   color -= 30;
 
   if((color <0)||(color > 7))
@@ -2070,6 +2147,9 @@ scr_back_color(int color)
 {
   XColor new_color;
 
+#ifdef DEBUG
+  check_text("set_back_color");  
+#endif
   color -= 40;
 
   if((color <0)||(color > 7))
