@@ -29,6 +29,10 @@
 #include "sbar.h"
 #include "debug.h"
 
+#ifdef GREEK_KBD	
+#include "grkelot.h"
+#endif
+
 extern WindowInfo MyWinInfo;
 
 #define XVT_CLASS	"XTerm"
@@ -88,8 +92,9 @@ extern KeySym BigFontKeysym;
 extern KeySym SmallFontKeysym;
 extern KeySym PageUpKeysym;
 extern KeySym PageDownKeysym;
-#ifdef GREEK_ELOT_KBD
+#ifdef GREEK_KBD
 extern KeySym GreekSwitchKeysym; 
+extern int GreekMode;
 #endif
 
 XSizeHints sizehints = {
@@ -227,6 +232,16 @@ void init_display(int argc,char **argv)
 	    MetaHandling = 0;
 	  i++;
 	}
+#ifdef GREEK_KBD
+      else if (strcmp(argv[i],"-grk9")==0)
+        {
+          GreekMode = GREEK_ELOT928;
+        }
+      else if (strcmp(argv[i],"-grk4")==0)
+        {
+          GreekMode = GREEK_IBM437;
+        }
+#endif       
       else if((strcmp(argv[i],"-secure")==0)&&(i+1<argc))
 	SecureKeysym = XStringToKeysym(argv[++i]);
       else if((strcmp(argv[i],"-bigfont")==0)&&(i+1<argc))
@@ -245,7 +260,7 @@ void init_display(int argc,char **argv)
 	}
       else 
 	{
-	  fprintf(stderr,"rxvt Version 2.03\n\n");
+	  fprintf(stderr,"rxvt Version 2.08\n\n");
 	  fprintf(stderr,"Permitted arguments are:\n");
 	  fprintf(stderr,
 "-e <command> <arg> ...	execute command with ars - must be last argument\n");
@@ -303,9 +318,12 @@ void init_display(int argc,char **argv)
 "-bigfont <keysym>      use hot key alt-keysym to switch to a bigger font\n");
 	  fprintf(stderr,
 "-small <keysym>        use hot key alt-keysym to switch to a smaller font\n");
-
-
-
+#ifdef GREEK_KBD
+	  fprintf(stderr,
+"-grk9                  greek kbd = ELOT 928 (default)\n");
+	  fprintf(stderr,
+"-grk4                  greek kbd = IBM 437\n");
+#endif
 	  clean_exit(1);
 	}
     }
